#if 0
#include <stdio.h>
#include "command.h"
#include "dosdebug.h"

void dump_words(const WORDS *list, const char *file_name, const char *mode)
{
  if (list && file_name && mode)
  {
    FILE f = fopen(file_name, mode);
    if (f)
    {
      WORD_LIST *words = list;
      int i;
      for (i = 0; words; words = words->next, i++)
      {
        WORD_DESC *word = words->word;
        fprintf(f,"word[%i] = %s\n", i, word->word);
      }
      fprintf(f, "-----------------------------\n\n");
      fclose(f);
    }
  }
}

void dump_command(const COMMAND *command, const char *file_name, const char *mode)
{
  if (command && file_name && mode)
  {
    FILE f = fopen(file_name, mode);
    if (f)
    {
      fprintf(f, "-----------------------------\n\n");
      fclose(f);
    }
  }
}
#else

#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <string.h>
#include <unistd.h>
#include "dosdebug.h"
#include "xmalloc.h"


#define CHECK_ENV_VALUE(s)    ((s) && ((s)[0] == 'Y' || (s)[0] == 'y') && ((s)[1] == 'E' || (s)[1] == 'e') && ((s)[2] == 'S' || (s)[2] == 's'))
#define PIPE_LOG_BUFFER_SIZE  (256 * 1024)

#define PIPE_LOG_FILENAME     "bshpipe0.log"

static bool delete_pipe_log_file = true;


void
log_pipe_creation(const int index, const struct pipe_t *pipe_line)
{
  if (log_pipe_files)
  {
    char *log_line;
    int line_length;
    if (previous_script_name != bash_script_name)
    {
      line_length = asprintf(&log_line, "\n\n"
                                        "================================================================================\n"
                                        "                    script: %s\n"
                                        "================================================================================\n"
                                        "\n\n", (bash_script_name));
      if (EOF != line_length)
      {
        const int consumed = pipe_log_buffer_ptr - pipe_log_buffer;
        if (PIPE_LOG_BUFFER_SIZE <= line_length + consumed)
        {
          FILE *pipe_log_file = fopen(pipe_log_filename, "a");
          if (!pipe_log_file)
          {
            free(log_line);
            return;
          }
          else
          {
            *pipe_log_buffer_ptr = '\0';
            fprintf(pipe_log_file, "%s", pipe_log_buffer);
            fclose(pipe_log_file);
            pipe_log_buffer_ptr = pipe_log_buffer;
          }
        }
        memcpy(pipe_log_buffer_ptr, log_line, line_length);
        pipe_log_buffer_ptr += line_length;
        free(log_line);
      }
      if (previous_script_name)
        xfree(previous_script_name);
      previous_script_name = bash_script_name;
    }

    if (bash_script_line_buffer)
    {
      line_length = asprintf(&log_line, "\n"
                                        "--------------------------------------------------------------------------------\n"
                                        "                    script line: %s\n"
                                        "--------------------------------------------------------------------------------\n"
                                        "\n", (bash_script_line_buffer));
      if (EOF != line_length)
      {
        const int consumed = pipe_log_buffer_ptr - pipe_log_buffer;
        if (PIPE_LOG_BUFFER_SIZE <= line_length + consumed)
        {
          FILE *pipe_log_file = fopen(pipe_log_filename, "a");
          if (!pipe_log_file)
          {
            free(log_line);
            return;
          }
          else
          {
            *pipe_log_buffer_ptr = '\0';
            fprintf(pipe_log_file, "%s", pipe_log_buffer);
            fclose(pipe_log_file);
            pipe_log_buffer_ptr = pipe_log_buffer;
          }
        }
        memcpy(pipe_log_buffer_ptr, log_line, line_length);
        pipe_log_buffer_ptr += line_length;
        free(log_line);
      }
    }

    line_length = asprintf(&log_line, "creating pipe #%d: %s\n"
                                      "  read  fd: %d\n"
                                      "  write fd: %d\n"
                                      "  bckup fd: %d\n",
                                      (index), pipe_line->name,
                                      pipe_line->fds[READ_END],
                                      pipe_line->fds[WRITE_END],
                                      pipe_line->fds[BACK_UP]);
    if (EOF != line_length)
    {
      const int consumed = pipe_log_buffer_ptr - pipe_log_buffer;
      if (PIPE_LOG_BUFFER_SIZE <= line_length + consumed)
      {
        FILE *pipe_log_file = fopen(pipe_log_filename, "a");
        if (!pipe_log_file)
        {
          free(log_line);
          return;
        }
        else
        {
          *pipe_log_buffer_ptr = '\0';
          fprintf(pipe_log_file, "%s", pipe_log_buffer);
          fclose(pipe_log_file);
          pipe_log_buffer_ptr = pipe_log_buffer;
        }
      }
      memcpy(pipe_log_buffer_ptr, log_line, line_length);
      pipe_log_buffer_ptr += line_length;
      free(log_line);
    }
  }

  delete_pipe_log_file = false;
}

void
log_pipe_operation(const int index, const char *operation, const char *status, const struct pipe_t *pipe_line)
{
  if (log_pipe_files)
  {
    char *log_line;
    int line_length;
    line_length = asprintf(&log_line, "%s of pipe #%d: %s\n"
                                      "  read  fd: %d\n"
                                      "  write fd: %d\n"
                                      "  bckup fd: %d\n"
                                      "  status: %s\n",
                                      (operation), (index),
                                      pipe_line->name,
                                      pipe_line->fds[READ_END],
                                      pipe_line->fds[WRITE_END],
                                      pipe_line->fds[BACK_UP],
                                      (status));
    if (EOF != line_length)
    {
      const int consumed = pipe_log_buffer_ptr - pipe_log_buffer;
      if (PIPE_LOG_BUFFER_SIZE <= line_length + consumed)
      {
        FILE *pipe_log_file = fopen(pipe_log_filename, "a");
        if (!pipe_log_file)
        {
          free(log_line);
          return;
        }
        else
        {
          *pipe_log_buffer_ptr = '\0';
          fprintf(pipe_log_file, "%s", pipe_log_buffer);
          fclose(pipe_log_file);
          pipe_log_buffer_ptr = pipe_log_buffer;
        }
      }
      memcpy(pipe_log_buffer_ptr, log_line, line_length);
      pipe_log_buffer_ptr += line_length;
      free(log_line);
    }
  }
}

void
log_pipe_dump_used_fd_array(const bool *const fd_in_use)
{
  if (log_pipe_files)
  {
    int i;
    for (i = 20; i < OPEN_MAX; i++)
    {
      if (fd_in_use[i] == true)
      {
        char *log_line;
        int line_length;
        line_length = asprintf(&log_line, "    fd_in_use[%03d] = TRUE\n", i);
        if (EOF != line_length)
        {
          const int consumed = pipe_log_buffer_ptr - pipe_log_buffer;
          if (PIPE_LOG_BUFFER_SIZE <= line_length + consumed)
          {
            FILE *pipe_log_file = fopen(pipe_log_filename, "a");
            if (!pipe_log_file)
            {
              free(log_line);
              return;
            }
            else
            {
              *pipe_log_buffer_ptr = '\0';
              fprintf(pipe_log_file, "%s", pipe_log_buffer);
              fclose(pipe_log_file);
              pipe_log_buffer_ptr = pipe_log_buffer;
            }
          }
          memcpy(pipe_log_buffer_ptr, log_line, line_length);
          pipe_log_buffer_ptr += line_length;
          free(log_line);
        }
      }
    }
  }
}

void
create_log_pipe_files(void)
{
  char *env_variable;

  /*  For debuging purposes only.  */
  env_variable = getenv("BASH_KEEP_PIPE_FILES");
  keep_pipe_files = CHECK_ENV_VALUE(env_variable) ? true : false;
  env_variable = getenv("BASH_LOG_PIPE_FILES");
  log_pipe_files = CHECK_ENV_VALUE(env_variable) ? true : false;

  if (log_pipe_files)
  {
    int length;
    char *filename, *tmpdir = "/dev/env/TMPDIR";
    if (access(tmpdir, D_OK))
    {
      tmpdir = "/dev/env/TMP";
      if (access(tmpdir, D_OK))
      {
        tmpdir = "/dev/env/TEMP";
        if (access(tmpdir, D_OK))
        {
          tmpdir = P_tmpdir;
          if (access(tmpdir, D_OK))
            tmpdir = ".";
        }
      }
    }
    filename = malloc(length = strlen(tmpdir) + sizeof("/"PIPE_LOG_FILENAME));
    if (!filename)
      log_pipe_files = false;
    else
    {
      FILE *pipe_log_file;
      char *numeric_tail = filename + length - sizeof("0.log");
      char *dot = numeric_tail + 1;
      int rc;

      sprintf(filename, "%s/%s", tmpdir, PIPE_LOG_FILENAME);
      if (0 == (rc = access(filename, F_OK)))
      {
        int i;
        for (i = 0; 0 == rc && i < 16777216; numeric_tail--)
        {
          do {
            sprintf(numeric_tail, "%x", ++i);
            *dot = '.';
          } while (0 == (rc = access(filename, F_OK)) && i % 15);
        }

        if (i == 16777216)
        {
          free(filename);
          log_pipe_files = false;
          return;
        }
      }
      pipe_log_file = fopen(filename, "w");
      if (!pipe_log_file)
      {
        free(filename);
        log_pipe_files = false;
        return;
      }
      fclose(pipe_log_file);

      pipe_log_buffer = malloc(PIPE_LOG_BUFFER_SIZE);
      if (!pipe_log_buffer)
        log_pipe_files = false;
      else
      {
        pipe_log_buffer_ptr = pipe_log_buffer;
        pipe_log_filename = filename;
        log_pipe_files = true;
      }
    }
  }
}

void
close_log_pipe_files(void)
{
  if (delete_pipe_log_file)
    remove(pipe_log_filename);
  else if (log_pipe_files && pipe_log_filename)
  {
    FILE *pipe_log_file = fopen(pipe_log_filename, "a");
    if (pipe_log_file)
    {
      *pipe_log_buffer_ptr = '\0';
      fprintf(pipe_log_file, "%s", pipe_log_buffer);
      fclose(pipe_log_file);
    }
  }
}
#endif
